@@ -2,7 +2,8 @@
 
 __docformat__ = 'restructuredtext'
 
-from .version import __version__
+# Imported to set singularity/apptainer version commands at init
+import datalad_container.extractors._load_singularity_versions  # noqa
 
 # defines a datalad command suite
 # this symbold must be identified as a setuptools entrypoint
@@ -45,3 +46,22 @@ command_suite = (
         )
     ]
 )
+
+from os.path import join as opj
+
+from datalad.support.constraints import EnsureStr
+from datalad.support.extensions import register_config
+
+register_config(
+    'datalad.containers.location',
+    'Container location',
+    description='path within the dataset where to store containers',
+    type=EnsureStr(),
+    default=opj(".datalad", "environments"),
+    dialog='question',
+    scope='dataset',
+)
+
+from . import _version
+
+__version__ = _version.get_versions()['version']
